/*
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package com.l2jfree.gameserver.model.actor.instance;

import com.l2jfree.gameserver.ai.CtrlIntention;
import com.l2jfree.gameserver.model.actor.L2Npc;
import com.l2jfree.gameserver.templates.chars.L2NpcTemplate;

 /**
 * @author squeezed
 */
public final class L2DecoInstance extends L2Npc
{
    /**
     * Constructor of L2DecoInstance (use L2Character and L2NpcInstance constructor).<BR><BR>
     *
     * <B><U> Actions</U> :</B><BR><BR>
     * <li>Call the L2Character constructor to set the _template of the L2DecoInstance (copy skills from template to object and link _calculators to NPC_STD_CALCULATOR) </li>
     * <li>Set the name of the L2DecoInstance</li>
     * <li>Create a RandomAnimation Task that will be launched after the calculated delay if the server allow it </li><BR><BR>
     *
     * @param objectId Identifier of the object to initialized
     * @param L2NpcTemplate Template to apply to the NPC
     */
    public L2DecoInstance(int objectId, L2NpcTemplate template)
    {
        super(objectId, template);
    }

    /**
     * Manage actions when a player click on the L2DecoInstance.<BR><BR>
     *
     * <B><U> Actions</U> :</B><BR><BR>
     * <li>Set the L2NpcInstance as target of the L2PcInstance player (if necessary)</li>
     * <li>Send a Server->Client packet MyTargetSelected to the L2PcInstance player (display the select window)</li>
     * <li>Send a Server->Client packet ValidateLocation to correct the L2NpcInstance position and heading on the client </li><BR><BR>
     *
     * <B><U> Example of use </U> :</B><BR><BR>
     * <li> Client packet : Action, AttackRequest</li><BR><BR>
     *
     * @param player The L2PcInstance that start an action on the L2DecoInstance
     *
     */
    @Override
    public void onAction(L2PcInstance player)
    {
        if (getObjectId() != player.getTargetId())
        {
            // Set the player AI Intention to AI_INTENTION_IDLE
            player.getAI().setIntention(CtrlIntention.AI_INTENTION_IDLE, null);

            // Set the target of the L2PcInstance player
            player.setTarget(this);
        }
    }
}
